@@ -46,7 +46,7 @@ class EventManager:
 
     def __init__(self):
 
-        self._listenerCounts = {}
+        self._scriptListenerCounts = {}
         self.registry = pyatspi.Registry
         self._enqueueCount = 0
         self._dequeueCount = 0
@@ -67,38 +67,15 @@ class EventManager:
 
         self._registerListener("window:activate")
         self._registerListener("window:deactivate")
-        self._registerListener("object:children-changed:remove")
-
-        win = orca_state.activeScript.utilities.activeWindow()
-        if win:
-            # Generate a fake window activation event so the application
-            # can tell the user about itself.
-            #
-            class _FakeEvent:
-                def __init__(self, source, eventType,
-                             detail1, detail2, any_data):
-                    self.source = source
-                    self.type = eventType
-                    self.detail1 = detail1
-                    self.detail2 = detail2
-                    self.any_data = any_data
-                    self.host_application = None
-
-            class _FakeData:
-                def __init__(self):
-                    pass
-                def value(self):
-                    return None
-
-            fe = _FakeEvent(win, "window:activate", 0, 0, _FakeData())
-            self._enqueue(fe)
+        self._registerListener("object:children-changed")
+        self._registerListener("mouse:button")
 
     def deactivate(self):
         """Called when this event manager is deactivated."""
 
-        for eventType in self._listenerCounts.keys():
+        for eventType in self._scriptListenerCounts.keys():
             self.registry.deregisterEventListener(self._enqueue, eventType)
-        self._listenerCounts = {}
+        self._scriptListenerCounts = {}
 
     def _ignore(self, event):
         """Returns True if this event should be ignored."""
@@ -297,11 +274,11 @@ class EventManager:
         - eventType: the event type.
         """
 
-        if eventType in self._listenerCounts:
-            self._listenerCounts[eventType] += 1
+        if eventType in self._scriptListenerCounts:
+            self._scriptListenerCounts[eventType] += 1
         else:
             self.registry.registerEventListener(self._enqueue, eventType)
-            self._listenerCounts[eventType] = 1
+            self._scriptListenerCounts[eventType] = 1
 
     def _deregisterListener(self, eventType):
         """Tells this module to stop listening for the given event type.
@@ -310,17 +287,17 @@ class EventManager:
         - eventType: the event type.
         """
 
-        if not eventType in self._listenerCounts:
+        if not eventType in self._scriptListenerCounts:
             return
 
-        self._listenerCounts[eventType] -= 1
-        if self._listenerCounts[eventType] == 0:
+        self._scriptListenerCounts[eventType] -= 1
+        if self._scriptListenerCounts[eventType] == 0:
             self.registry.deregisterEventListener(self._enqueue, eventType)
-            del self._listenerCounts[eventType]
+            del self._scriptListenerCounts[eventType]
 
-    def registerListeners(self, script):
-        """Tells the FocusTrackingPresenter to listen for all
-        the event types of interest to the script.
+    def registerScriptListeners(self, script):
+        """Tells the event manager to start listening for all the event types
+        of interest to the script.
 
         Arguments:
         - script: the script.
@@ -329,9 +306,9 @@ class EventManager:
         for eventType in script.listeners.keys():
             self._registerListener(eventType)
 
-    def deregisterListeners(self, script):
-        """Tells the FocusTrackingPresenter to stop listening for all the
-        event types of interest to the script.
+    def deregisterScriptListeners(self, script):
+        """Tells the event manager to stop listening for all the event types
+        of interest to the script.
 
         Arguments:
         - script: the script.
@@ -340,6 +317,41 @@ class EventManager:
         for eventType in script.listeners.keys():
             self._deregisterListener(eventType)
 
+    def registerModuleListeners(self, listeners):
+        """Register the listeners on behalf of the caller."""
+
+        for eventType, function in listeners.items():
+            self.registry.registerEventListener(function, eventType)
+
+    def deregisterModuleListeners(self, listeners):
+        """Deegister the listeners on behalf of the caller."""
+
+        for eventType, function in listeners.items():
+            self.registry.deregisterEventListener(function, eventType)
+
+    def registerKeystrokeListener(self, function, mask=None, kind=None):
+        """Register the keystroke listener on behalf of the caller."""
+
+        if mask == None:
+            mask = range(256)
+
+        if kind == None:
+            kind = (pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT)
+
+        self.registry.registerKeystrokeListener(function, mask=mask, kind=kind)
+
+    def deregisterKeystrokeListener(self, function, mask=None, kind=None):
+        """Deregister the keystroke listener on behalf of the caller."""
+
+        if mask == None:
+            mask = range(256)
+
+        if kind == None:
+            kind = (pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT)
+
+        self.registry.deregisterKeystrokeListener(
+            function, mask=mask, kind=kind)
+
     def _processInputEvent(self, event):
         """Processes the given input event based on the keybinding from the
         currently-active script.
@@ -397,6 +409,19 @@ class EventManager:
         of debugging).
         """
 
+        if not event.source:
+            return False, "event.source? What event.source??"
+
+        role = state = None
+        try:
+            role = event.source.getRole()
+        except LookupError:
+            return False, "LookupError getting event.source's role"
+        try:
+            state = event.source.getState()
+        except LookupError:
+            return False, "LookupError getting event.source's state"
+        
         if not script:
             script = self._getScriptForEvent(event)
 
@@ -421,8 +446,8 @@ class EventManager:
         # This condition appears with gnome-screensave-dialog.
         # See bug 530368.
         if eType.startswith('object:state-changed:showing') \
-           and event.source.getRole() == pyatspi.ROLE_PANEL \
-           and event.source.getState().contains(pyatspi.STATE_MODAL):
+           and role == pyatspi.ROLE_PANEL \
+           and state.contains(pyatspi.STATE_MODAL):
             return True, "Modal panel is showing."
 
         return False, "No reason found to activate a different script."
@@ -443,6 +468,8 @@ class EventManager:
                     _scriptManager.reclaimScripts()
                     if settings.debugMemoryUsage:
                         orca.cleanupGarbage()
+                    if not event.source.childCount:
+                        orca.shutdown()
                     return
             except (LookupError, RuntimeError):
                 # If we got this error here, we'll get it again when we
