@@ -34,7 +34,6 @@ import re
 import signal
 import sys
 import time
-import unicodedata
 import shutil
 
 from gi.repository.Gio import Settings
@@ -541,7 +540,6 @@ if settings.useDBus:
 
 import braille
 import httpserver
-import keynames
 import keybindings
 import orca_state
 import speech
@@ -549,8 +547,6 @@ import notification_messages
 
 from input_event import BrailleEvent
 from input_event import KeyboardEvent
-from input_event import MouseButtonEvent
-from input_event import keyEventToString
 
 import gc
 if settings.debugMemoryUsage:
@@ -660,48 +656,6 @@ def setLocusOfFocus(event, obj, notifyScript=True, force=False):
         orca_state.activeScript.locusOfFocusChanged(
             event, oldLocusOfFocus, orca_state.locusOfFocus)
 
-def _onChildrenChanged(e):
-    """Tracks children-changed events on the desktop to determine when
-    apps start and stop.
-
-    Arguments:
-    - e: at-spi event from the at-api registry
-    """
-
-    desktop = pyatspi.Registry.getDesktop(0)
-    if e.source == desktop:
-
-        # If the desktop is empty, the user has logged out-- shutdown Orca
-        #
-        try:
-            if desktop.childCount == 0:
-                if not _settingsManager.getSetting('onlySpeakDisplayedText'):
-                    speech.speak(_("Goodbye."))
-                shutdown()
-                return
-        except: # could be a CORBA.COMM_FAILURE
-            debug.printException(debug.LEVEL_FINEST)
-            shutdown()
-            return
-
-def _onMouseButton(e):
-    """Tracks mouse button events, stopping any speech in progress.
-
-    Arguments:
-    - e: at-spi event from the at-api registry
-    """
-
-    mouse_event = MouseButtonEvent(e)
-    orca_state.lastInputEvent = mouse_event
-
-    # A mouse button event looks like: mouse:button:1p, where the
-    # number is the button number and the 'p' is either 'p' or 'r',
-    # meaning pressed or released.  We only want to stop speech on
-    # button presses.
-    #
-    if mouse_event.pressed:
-        speech.stop()
-
 ########################################################################
 #                                                                      #
 # DEBUG support.                                                       #
@@ -735,26 +689,6 @@ def cycleDebugLevel(script=None, inputEvent=None):
 
     return True
 
-def exitLearnMode(self, inputEvent=None):
-    """Turns learn mode off.
-
-    Returns True to indicate the input event has been consumed.
-    """
-
-    # Translators: Orca has a "Learn Mode" that will allow
-    # the user to type any key on the keyboard and hear what
-    # the effects of that key would be.  The effects might
-    # be what Orca would do if it had a handler for the
-    # particular key combination, or they might just be to
-    # echo the name of the key if Orca doesn't have a handler.
-    # Exiting learn mode puts the user back in normal operating
-    # mode.
-    #
-    message = _("Exiting learn mode.")
-    orca_state.activeScript.presentMessage(message)
-    settings.learnModeEnabled = False
-    return True
-
 def exitListShortcutsMode(self, inputEvent=None):
     """Turns list shortcuts mode off.
 
@@ -784,360 +718,15 @@ def exitListShortcutsMode(self, inputEvent=None):
 #                                                                      #
 ########################################################################
 
-# Keybindings that Orca itself cares about.
-#
-_keyBindings = None
-
-# True if the orca modifier key is currently pressed.
-#
 _orcaModifierPressed = False
 
-def isPrintableKey(event_string):
-    """Return an indication of whether this is an alphanumeric or
-       punctuation key.
-
-    Arguments:
-    - event: the event string
-
-    Returns True if this is an alphanumeric or punctuation key.
-    """
-
-    if event_string == "space":
-        reply = True
-    else:
-        unicodeString = event_string.decode("UTF-8")
-        reply = (len(unicodeString) == 1) \
-                and (unicodeString.isalnum() or unicodeString.isspace()
-                or unicodedata.category(unicodeString)[0] in ('P', 'S'))
-    debug.println(debug.LEVEL_FINEST,
-                  "orca.isPrintableKey: returning: %s" % reply)
-    return reply
-
-def isModifierKey(event_string):
-    """Return an indication of whether this is a modifier key.
-
-    Arguments:
-    - event: the event string
-
-    Returns True if this is a modifier key
-    """
-
-    modifierKeys = [ 'Alt_L', 'Alt_R', 'Control_L', 'Control_R', \
-                     'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R' ]
-
-    if not orca_state.bypassNextCommand:
-        orcaModifiers = settings.orcaModifierKeys
-        try:
-            orcaModifiers = map(lambda x: x.encode('UTF-8'), orcaModifiers)
-        except (UnicodeDecodeError, UnicodeEncodeError):
-            pass
-        modifierKeys.extend(orcaModifiers)
-
-    if isinstance(event_string, unicode):
-        event_string.encode('UTF-8')
-
-    reply = event_string in modifierKeys
-    debug.println(debug.LEVEL_FINEST,
-                  "orca.isModifierKey: returning: %s" % reply)
-    return reply
-
-def isLockingKey(event_string):
-    """Return an indication of whether this is a locking key.
-
-    Arguments:
-    - event: the event string
-
-    Returns True if this is a locking key.
-    """
-
-    lockingKeys = [ "Caps_Lock", "Num_Lock", "Scroll_Lock" ]
-
-    reply = event_string in lockingKeys
-    if not orca_state.bypassNextCommand:
-        reply = reply and not event_string in settings.orcaModifierKeys
-    debug.println(debug.LEVEL_FINEST,
-                  "orca.isLockingKey: returning: %s" % reply)
-    return reply
-
-def isFunctionKey(event_string):
-    """Return an indication of whether this is a function key.
-
-    Arguments:
-    - event: the event string
-
-    Returns True if this is a function key.
-    """
-
-    # [[[TODO:richb - what should be done about the function keys on the left
-    #    side of my Sun keyboard and the other keys (like Scroll Lock), which
-    #    generate "Fn" key events?]]]
-
-    functionKeys = [ "F1", "F2", "F3", "F4", "F5", "F6",
-                     "F7", "F8", "F9", "F10", "F11", "F12" ]
-
-    reply = event_string in functionKeys
-    debug.println(debug.LEVEL_FINEST,
-                  "orca.isFunctionKey: returning: %s" % reply)
-    return reply
-
-def isActionKey(event_string):
-    """Return an indication of whether this is an action key.
-
-    Arguments:
-    - event: the event string
-
-    Returns True if this is an action key.
-    """
-
-    actionKeys = [ "Return", "Escape", "Tab", "BackSpace", "Delete",
-                   "Page_Up", "Page_Down", "Home", "End" ]
-
-    reply = event_string in actionKeys
-    debug.println(debug.LEVEL_FINEST,
-                  "orca.isActionKey: returning: %s" % reply)
-    return reply
-
-def isNavigationKey(event_string):
-    """Return an indication of whether this is a navigation (arrow) key
-    or if the user has the Orca modifier key held done.
-
-    Arguments:
-    - event: the event string
-    - modifiers: key modifiers state
-
-    Returns True if this is a navigation key.
-    """
-
-    navigationKeys = [ "Left", "Right", "Up", "Down" ]
-
-    reply = event_string in navigationKeys
-    if not reply and not orca_state.bypassNextCommand:
-        reply = _orcaModifierPressed
-
-    debug.println(debug.LEVEL_FINEST,
-                  "orca.isNavigationKey: returning: %s" % reply)
-    return reply
-
-def isDiacriticalKey(event_string):
-    """Return an indication of whether this is a non-spacing diacritical key
-
-    Arguments:
-    - event: the event string
-    - modifiers: key modifiers state
-
-    Returns True if this is a non-spacing diacritical key
-    """
-
-    reply = event_string.startswith("dead_")
-
-    debug.println(debug.LEVEL_FINEST,
-                  "orca.isDiacriticalKey: returning: %s" % reply)
-    return reply
-
-class KeyEventType:
-    """Definition of available key event types."""
-
-    # An alphanumeric or punctuation key event.
-    PRINTABLE = 'PRINTABLE'
-
-    # A modifier key event.
-    MODIFIER = 'MODIFIER'
-
-    # A locking key event.
-    LOCKING = 'LOCKING'
-
-    # A locking key lock event.
-    LOCKING_LOCKED = 'LOCKING_LOCKED'
-
-    # A locking key unlock event.
-    LOCKING_UNLOCKED = 'LOCKING_UNLOCKED'
-
-    # A function key event.
-    FUNCTION = 'FUNCTION'
-
-    # An action key event.
-    ACTION = 'ACTION'
-
-    # A navigation key event.
-    NAVIGATION = 'NAVIGATION'
-
-    # A diacritical key event.
-    DIACRITICAL = 'DIACRITICAL'
-
-    def __init__(self):
-        pass
-
-def keyEcho(event):
-    """If the keyEcho setting is enabled, check to see what type of key
-    event it is and echo it via speech, if the user wants that type of
-    key echoed.
-
-    Uppercase keys will be spoken using the "uppercase" voice style,
-    whereas lowercase keys will be spoken using the "default" voice style.
-
-    Arguments:
-    - event: an AT-SPI DeviceEvent
-    """
-
-    # If this keyboard event was for an object like a password text
-    # field, then don't echo it.
-    #
-    try:
-        role = orca_state.locusOfFocus.getRole()
-    except (LookupError, RuntimeError):
-        debug.println(debug.LEVEL_SEVERE,
-                      "orca.keyEcho() - locusOfFocus no longer exists")
-        setLocusOfFocus(None, None, False)
-        return False
-    except:
-        pass
-    else:
-        if role == pyatspi.ROLE_PASSWORD_TEXT:
-            return False
-
-    event_string = event.event_string
-    debug.println(debug.LEVEL_FINEST,
-                  "orca.keyEcho: string to echo: %s" % event_string)
-
-    # If key echo is enabled, then check to see what type of key event
-    # it is and echo it via speech, if the user wants that type of key
-    # echoed.
-    #
-    if settings.enableKeyEcho:
-
-        if isModifierKey(event_string):
-            if not settings.enableModifierKeys:
-                return False
-            eventType = KeyEventType.MODIFIER
-
-        elif isNavigationKey(event_string):
-            if not settings.enableNavigationKeys:
-                return False
-            eventType = KeyEventType.NAVIGATION
-
-        elif isDiacriticalKey(event_string):
-            if not settings.enableDiacriticalKeys:
-                return False
-            eventType = KeyEventType.DIACRITICAL
-
-        elif isPrintableKey(event_string):
-            if not (settings.enablePrintableKeys \
-                    or settings.enableEchoByCharacter):
-                return False
-            eventType = KeyEventType.PRINTABLE
-
-        elif isLockingKey(event_string):
-            if not settings.enableLockingKeys:
-                return False
-            eventType = KeyEventType.LOCKING
-
-            modifiers = event.modifiers
-
-            brailleMessage = None
-            if event_string == "Caps_Lock":
-                if modifiers & (1 << pyatspi.MODIFIER_SHIFTLOCK):
-                    eventType = KeyEventType.LOCKING_UNLOCKED
-                    brailleMessage = keynames.getKeyName(event_string) \
-                                     + " " + _("off")
-                else:
-                    eventType = KeyEventType.LOCKING_LOCKED
-                    brailleMessage = keynames.getKeyName(event_string) \
-                                     + " " + _("on")
-
-            elif event_string == "Num_Lock":
-                if modifiers & (1 << pyatspi.MODIFIER_NUMLOCK):
-                    eventType = KeyEventType.LOCKING_UNLOCKED
-                else:
-                    eventType = KeyEventType.LOCKING_LOCKED
-
-            if brailleMessage:
-                braille.displayMessage(brailleMessage,
-                                       flashTime=settings.brailleFlashTime)
-
-        elif isFunctionKey(event_string):
-            if not settings.enableFunctionKeys:
-                return False
-            eventType = KeyEventType.FUNCTION
-
-        elif isActionKey(event_string):
-            if not settings.enableActionKeys:
-                return False
-            eventType = KeyEventType.ACTION
-
-        else:
-            debug.println(debug.LEVEL_FINEST,
-                  "orca.keyEcho: event string not handled: %s" % event_string)
-            return False
-
-        # We keep track of the time as means to let others know that
-        # we are probably echoing a key and should not be interrupted.
-        #
-        orca_state.lastKeyEchoTime = time.time()
-
-        # Before we echo printable keys, let's try to make sure the
-        # character echo isn't going to also echo something.
-        #
-        characterEcho = \
-            eventType == KeyEventType.PRINTABLE \
-            and not _orcaModifierPressed \
-            and orca_state.activeScript \
-            and orca_state.activeScript.utilities.willEchoCharacter(event)
-
-        # One last check for echoing -- a PRINTABLE key may have squeaked
-        # through due to the enableEchoByCharacter check above.  We only
-        # want to echo PRINTABLE keys if enablePrintableKeys is True.
-        #
-        if not (characterEcho or
-                (eventType == KeyEventType.PRINTABLE \
-                 and not settings.enablePrintableKeys)):
-            debug.println(debug.LEVEL_FINEST,
-                          "orca.keyEcho: speaking: %s" % event_string)
-            speech.speakKeyEvent(event_string, eventType)
-            return True
-        elif characterEcho:
-            debug.println(debug.LEVEL_FINEST,
-                          "orca.keyEcho: letting character echo handle: %s" \
-                          % event_string)
-            return False
-
-def _setClickCount(inputEvent):
-    """Sets the count of the number of clicks a user has made to one
-    of the non-modifier keys on the keyboard.  Note that this looks at
-    the event_string (keysym) instead of hw_code (keycode) because
-    the Java platform gives us completely different keycodes for keys.
-
-    Arguments:
-    - inputEvent: the current input event.
-    """
-
-    lastInputEvent = orca_state.lastNonModifierKeyEvent
-
-    if inputEvent.type == pyatspi.KEY_RELEASED_EVENT:
-        pass
-    elif not isinstance(inputEvent, KeyboardEvent):
-        orca_state.clickCount = 0
-    elif not isinstance(lastInputEvent, KeyboardEvent):
-        orca_state.clickCount = 1
-    elif (lastInputEvent.event_string != inputEvent.event_string) or \
-         (lastInputEvent.modifiers != inputEvent.modifiers):
-        orca_state.clickCount = 1
-    elif (inputEvent.time - lastInputEvent.time) < \
-           settings.doubleClickTimeout:
-        # Cap the possible number of clicks at 3.
-        #
-        if orca_state.clickCount < 3:
-            orca_state.clickCount += 1
-    else:
-        orca_state.clickCount = 1
-
 def _processKeyCaptured(event):
     """Called when a new key event arrives and orca_state.capturingKeys=True.
     (used for key bindings redefinition)
     """
 
     if event.type == 0:
-        if isModifierKey(event.event_string) \
-           or isLockingKey(event.event_string):
+        if event.isModifierKey() or event.isLockingKey():
             return True
         else:
             # We want the keyname rather than the printable character.
@@ -1167,7 +756,7 @@ def _processKeyCaptured(event):
 
 def _processKeyboardEvent(event):
     """The primary key event handler for Orca.  Keeps track of various
-    attributes, such as the lastInputEvent.  Also calls keyEcho as well
+    attributes, such as the lastInputEvent.  Also does key echo as well
     as any local keybindings before passing the event on to the active
     script.  This method is called synchronously from the AT-SPI registry
     and should be performant.  In addition, it must return True if it has
@@ -1180,180 +769,66 @@ def _processKeyboardEvent(event):
     """
     global _orcaModifierPressed
 
-    # Input methods appear to play games with repeating events
-    # and also making up events with no timestamps.  We try
-    # to handle that here. See bug #589504.
-    #
-    if (event.timestamp == 0) \
-       or (event.timestamp == orca_state.lastInputEventTimestamp \
-           and orca_state.lastInputEvent \
-           and orca_state.lastInputEvent.hw_code == event.hw_code \
-           and orca_state.lastInputEvent.type == event.type):
-        debug.println(debug.LEVEL_FINE, keyEventToString(event))
+    # Weed out duplicate and otherwise bogus events.
+    keyboardEvent = KeyboardEvent(event)
+    debug.println(debug.LEVEL_FINE, keyboardEvent.toString())
+    if keyboardEvent.ignoreDueToTimestamp():
         debug.println(debug.LEVEL_FINE, "IGNORING EVENT DUE TO TIMESTAMP")
         return
 
-    orca_state.lastInputEventTimestamp = event.timestamp
-
-    # Log the keyboard event for future playback, if desired.
-    # Note here that the key event_string being output is
-    # exactly what we received.  The KeyboardEvent object,
-    # however, will translate the event_string for control
-    # characters to their upper case ASCII equivalent.
-    #
-    string = keyEventToString(event)
-    debug.printInputEvent(debug.LEVEL_FINE, string)
-
-    keyboardEvent = KeyboardEvent(event)
-
-    # Log the KeyboardEvent translated from the pyatspi keyboardEvent.
-    #
-    string = keyboardEvent.toString()
-    debug.printInputEvent(debug.LEVEL_FINE, string)
-
-    if keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
-        braille.killFlash()
-
-    # See if this is one of our special Orca modifier keys.
-    #
-    # Just looking at the keycode should suffice, but there is a
-    # "feature" in the Java Access Bridge where it chooses to emit
-    # Java platform-independent keycodes instead of the keycodes
-    # for the base platform:
-    #
-    # http://bugzilla.gnome.org/show_bug.cgi?id=106004
-    # http://bugzilla.gnome.org/show_bug.cgi?id=318615
-    #
-    # So...we need to workaround this problem.
-    #
-    # If you make the following expression True we will get a positive
-    # match for all keysyms associated with a given keysym specified
-    # as an Orca modifier key.
-    #
-    # For example, assume the Orca modifier is set to \ for some
-    # reason.  The key that has \ on it produces \ without the Shift
-    # key and | with the Shift key.  If the following expression is
-    # True, both the \ and | will be viewed as the Orca modifier.  If
-    # the following expression is False, only the \ will be viewed as
-    # the Orca modifier (i.e., Shift+\ will still function as the |
-    # character).  In general, I think we want to avoid sucking in all
-    # possible keysyms because it can have unexpected results.
-    #
-    if False:
-        allPossibleKeysyms = []
-        for keysym in settings.orcaModifierKeys:
-            allPossibleKeysyms.extend(keybindings.getAllKeysyms(keysym))
-    else:
-        allPossibleKeysyms = settings.orcaModifierKeys
-
-    try:
-        allPossibleKeysyms = \
-            map(lambda x: x.encode('UTF-8'), allPossibleKeysyms)
-    except (UnicodeDecodeError, UnicodeEncodeError):
-        pass
-
-    isOrcaModifier = allPossibleKeysyms.count(keyboardEvent.event_string) > 0
-
-    if event.type == pyatspi.KEY_PRESSED_EVENT:
-        # Key presses always interrupt speech.
-        #
-        speech.stop()
-
-        # We treat the Insert key as a modifier - so just swallow it and
-        # set our internal state.
-        #
-        if isOrcaModifier:
-            _orcaModifierPressed = True
-
-        # If learn mode is enabled, it will echo the keys.
-        #
-        if not settings.learnModeEnabled and \
-           orca_state.activeScript.echoKey(keyboardEvent):
-            try:
-                keyEcho(keyboardEvent)
-            except:
-                debug.printException(debug.LEVEL_SEVERE)
-
-    elif isOrcaModifier \
-        and (keyboardEvent.type == pyatspi.KEY_RELEASED_EVENT):
-        _orcaModifierPressed = False
-
+    # Figure out what we've got.
+    isOrcaModifier = keyboardEvent.isOrcaModifier()
+    isPressedEvent = keyboardEvent.isPressedKey()
+    if isOrcaModifier:
+        _orcaModifierPressed = isPressedEvent
     if _orcaModifierPressed:
-        keyboardEvent.modifiers = keyboardEvent.modifiers \
-                                  | settings.ORCA_MODIFIER_MASK
+        keyboardEvent.modifiers |= settings.ORCA_MODIFIER_MASK
 
+    # Update our state.
+    orca_state.lastInputEventTimestamp = event.timestamp
     orca_state.lastInputEvent = keyboardEvent
-
-    # If this is a key event for a non-modifier key, save a handle to it.
-    # This is needed to help determine user actions when a multi-key chord
-    # has been pressed, and we might get the key events in different orders.
-    # See comment #15 of bug #435201 for more details.  We also want to
-    # store the "click count" for the purpose of supporting keybindings
-    # with unique behaviors when double- or triple-clicked.
-    #
-    if not isModifierKey(keyboardEvent.event_string):
-        _setClickCount(keyboardEvent)
+    if not keyboardEvent.isModifierKey():
+        keyboardEvent.setClickCount()
         orca_state.lastNonModifierKeyEvent = keyboardEvent
 
-    # Orca gets first stab at the event.  Then, the presenter gets
-    # a shot. [[[TODO: WDW - might want to let the presenter try first?
-    # The main reason this is staying as is is that we may not want
-    # scripts to override fundamental Orca key bindings.]]]
-    #
-    consumed = False
-    try:
-        if orca_state.capturingKeys:
-            consumed = _processKeyCaptured(keyboardEvent)
-        else:
-            if settings.listShortcutsModeEnabled:
-                consumed = listShortcuts(keyboardEvent)
-            elif notification_messages.listNotificationMessagesModeEnabled:
-                consumed = notification_messages.listNotificationMessages(
-                    keyboardEvent)
-            if (not consumed):
-                consumed = _keyBindings.consumeKeyboardEvent( \
-                  None, keyboardEvent)
-            if (not consumed):
-                consumed = _eventManager.processKeyboardEvent(keyboardEvent)
-            if (not consumed) and settings.learnModeEnabled:
-                if keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
-                    clickCount = orca_state.activeScript.getClickCount()
-                    if isPrintableKey(keyboardEvent.event_string) \
-                       and clickCount == 2:
-                        orca_state.activeScript.phoneticSpellCurrentItem(\
-                            keyboardEvent.event_string)
-                    else:
-                        # Check to see if there are localized words to be
-                        # spoken for this key event.
-                        #
-                        braille.displayMessage(keyboardEvent.event_string)
-                        event_string = keyboardEvent.event_string
-                        event_string = keynames.getKeyName(event_string)
-                        speech.speak(event_string)
-                elif (event.type == pyatspi.KEY_RELEASED_EVENT) and \
-                     (keyboardEvent.event_string == "Escape"):
-                    exitLearnMode(keyboardEvent)
+    # Echo it based on what it is and the user's settings.
+    script = orca_state.activeScript
+    if script:
+        if isPressedEvent:
+            script.presentationInterrupt()
+        keyboardEvent.present()
+ 
+    # Special modes.
+    if not isPressedEvent and keyboardEvent.event_string == "Escape":
+        script.exitLearnMode(keyboardEvent)
+    if orca_state.capturingKeys:
+        return _processKeyCaptured(keyboardEvent)
+    if settings.listShortcutsModeEnabled:
+        return listShortcuts(keyboardEvent)
+    if notification_messages.listNotificationMessagesModeEnabled:
+        return notification_messages.listNotificationMessages(keyboardEvent)
+    if settings.learnModeEnabled:
+        if keyboardEvent.isPrintableKey() and not _orcaModifierPressed:
+            return True
 
-                consumed = True
+    # See if the event manager wants it (i.e. it is bound to a command.
+    if _eventManager.processKeyboardEvent(keyboardEvent):
+        return True
 
-            global _restoreOrcaKeys
-            if not consumed \
-               and keyboardEvent.type == pyatspi.KEY_RELEASED_EVENT:
-                if isOrcaModifier and orca_state.bypassNextCommand:
-                    _restoreXmodmap()
-                    _restoreOrcaKeys = True
-                elif _restoreOrcaKeys and not orca_state.bypassNextCommand:
-                    _createOrcaXmodmap()
-                    _restoreOrcaKeys = False
-
-            if not consumed \
-               and not isModifierKey(keyboardEvent.event_string) \
-               and keyboardEvent.type == pyatspi.KEY_PRESSED_EVENT:
-                orca_state.bypassNextCommand = False
-    except:
-        debug.printException(debug.LEVEL_SEVERE)
-
-    return consumed or isOrcaModifier
+    # Do any needed xmodmap crap.
+    global _restoreOrcaKeys
+    if not isPressedEvent:
+        if keyboardEvent.event_string in settings.orcaModifierKeys \
+           and orca_state.bypassNextCommand:
+            _restoreXmodmap()
+            _restoreOrcaKeys = True
+        elif _restoreOrcaKeys and not orca_state.bypassNextCommand:
+            _createOrcaXmodmap()
+            _restoreOrcaKeys = False
+    elif not keyboardEvent.isModifierKey():
+        orca_state.bypassNextCommand = False
+ 
+    return isOrcaModifier
 
 ########################################################################
 #                                                                      #
@@ -1721,7 +1196,7 @@ def listShortcuts(event):
         _("Use Up and Down Arrow to navigate the list. Press Escape to exit.")
 
     if event.type == pyatspi.KEY_PRESSED_EVENT:
-        clickCount = orca_state.activeScript.getClickCount()
+        clickCount = event.getClickCount()
         if (event.event_string == "1"):
             if not numShortcuts or orca_state.typeOfShortcuts != "default":
                 orca_state.listOfShortcuts = getListOfShortcuts("default")
@@ -1979,7 +1454,6 @@ def init(registry):
     """
 
     global _initialized
-    global _keyBindings
 
     if _initialized \
        and a11yAppSettings.get_boolean('screen-reader-enabled'):
@@ -1991,34 +1465,8 @@ def init(registry):
         signal.signal(signal.SIGALRM, settings.timeoutCallback)
         signal.alarm(settings.timeoutTime)
 
-    # Note that we have moved the Orca specific keybindings to the default
-    # script, so _keyBindings is currently empty. The logic is retained
-    # here, just in case we wish to reinstate them in the future.
-    #
-    _keyBindings = keybindings.KeyBindings()
-
-    # Create and load an app's script when it is added to the desktop
-    #
-    registry.registerEventListener(_onChildrenChanged,
-                                   "object:children-changed")
-
-    # We also want to stop speech when a mouse button is pressed.
-    #
-    registry.registerEventListener(_onMouseButton,
-                                   "mouse:button")
-
     loadUserSettings()
-
-    masks = []
-    mask = 0
-    # Cover all masks in 8 bits.
-    while mask <= 255:
-        masks.append(mask)
-        mask += 1
-    pyatspi.Registry.registerKeystrokeListener(
-        _processKeyboardEvent,
-        mask=masks,
-        kind=(pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT))
+    _eventManager.registerKeystrokeListener(_processKeyboardEvent)
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
@@ -2088,13 +1536,6 @@ def shutdown(script=None, inputEvent=None):
     #
     orca_state.activeScript.presentMessage(_("Goodbye."))
 
-    # Deregister our event listeners
-    #
-    pyatspi.Registry.deregisterEventListener(_onChildrenChanged,
-                                             "object:children-changed")
-    pyatspi.Registry.deregisterEventListener(_onMouseButton,
-                                             "mouse:button")
-
     _eventManager.deactivate()
     _scriptManager.deactivate()
 
@@ -2306,6 +1747,11 @@ def main():
         debug.printException(debug.LEVEL_SEVERE)
 
     showSplashGUI()
+    script = orca_state.activeScript
+    if script:
+        window = script.utilities.activeWindow()
+        if window and not orca_state.locusOfFocus:
+            setLocusOfFocus(None, window)
 
     # Check to see if the user wants the configuration GUI. It's
     # done here so that the user's existing preferences can be used
