@@ -71,7 +71,12 @@ class Script:
         self.app = app
 
         if app:
-            self.name = self.app.name
+            try:
+                self.name = self.app.name
+            except LookupError:
+                msg = 'script.__init__: LookupError trying to get app.name'
+                debug.println(debug.LEVEL_FINE, msg)
+                self.name = "default"
         else:
             self.name = "default"
 
@@ -91,10 +96,6 @@ class Script:
         self.pointOfReference = {}
         self.setupInputEventHandlers()
         self.keyBindings = self.getKeyBindings()
-        if settings.validateKeyBindings:
-            validation = self.keyBindings.validate()
-            if len(validation):
-                debug.println(debug.LEVEL_SEVERE, validation)
         self.brailleBindings = self.getBrailleBindings()
         self.app_pronunciation_dict = self.getPronunciations()
 
@@ -134,6 +135,16 @@ class Script:
         """
         return keybindings.KeyBindings()
 
+    def getToolkitKeyBindings(self):
+        """Returns the toolkit-specific keybindings for this script."""
+
+        return keybindings.KeyBindings()
+
+    def getAppKeyBindings(self):
+        """Returns the application-specific keybindings for this script."""
+
+        return keybindings.KeyBindings()
+
     def getKeyBindingsForInputHandler(self, inputEventHandler):
         """ Returns a KeyBindings object with the list of KeyBindings that
         matche the passed inputEventHandler as argument (at least the
@@ -243,21 +254,6 @@ class Script:
         """
         return where_am_I.WhereAmI(self)
 
-    def echoKey(self, keyEvent):
-        """Determine whether this script should echo the current key event.
-        Note that the keyEcho() method in orca.py will still take into
-        account whatever the user's various preferences for key echoing
-        are, which may override what is return by this echoKey() method.
-
-        Arguments:
-        - keyEvent - the key event
-
-        Returns an indication of whether a key echo event should be
-        allowed to happen for this script.
-        """
-
-        return True
-
     def getBookmarks(self):
         """Returns the "bookmarks" class for this script.
         """
@@ -316,13 +312,6 @@ class Script:
         """
         return
 
-    def getClickCount(self):
-        """Return the count of the number of clicks a user has made to one
-        of the keys on the keyboard.
-        """
-
-        return orca_state.clickCount
-
     def getSettings(self):
         """Returns the settings associated with this script, regardless of
         whether or not the script is active.
@@ -369,11 +358,23 @@ class Script:
         - event: the Event
         """
 
+        if not event.source:
+            msg = 'script.processObjectEvent: event.source went away'
+            debug.println(debug.LEVEL_FINE, msg)
+            return
+
+        try:
+            role = event.source.getRole()
+        except LookupError:
+            msg = 'script.processObjectEvent: LookupError getting role'
+            debug.println(debug.LEVEL_FINE, msg)
+            return
+
         # Check to see if we really want to process this event.
         #
         processEvent = (orca_state.activeScript == self \
                         or self.presentIfInactive)
-        if event.source.getRole() == pyatspi.ROLE_PROGRESS_BAR \
+        if role == pyatspi.ROLE_PROGRESS_BAR \
            and not processEvent \
            and settings.progressBarVerbosity == settings.PROGRESS_BAR_ALL:
             processEvent = True
