@@ -74,7 +74,6 @@ userCustomizableSettings = [
     "enableKeyEcho",
     "enablePrintableKeys",
     "enableModifierKeys",
-    "enableLockingKeys",
     "enableFunctionKeys",
     "enableActionKeys",
     "enableNavigationKeys",
@@ -197,11 +196,6 @@ DESKTOP_MODIFIER_KEYS = ["Insert", "KP_Insert"]
 LAPTOP_MODIFIER_KEYS  = ["Caps_Lock"]
 orcaModifierKeys      = DESKTOP_MODIFIER_KEYS
 
-# If True, some validation of the keybindings will be done to check for
-# missing keysyms and duplicate keys used for different operations.
-#
-validateKeyBindings = False
-
 # A new modifier to use (set by the press of any key in the
 # orcaModifierKeys list) to represent the Orca modifier.
 #
@@ -421,10 +415,6 @@ enablePrintableKeys     = True
 #
 enableModifierKeys      = True
 
-# If True and key echo is enabled, echo Locking keys.
-#
-enableLockingKeys       = True
-
 # If True and key echo is enabled, echo Function keys.
 #
 enableFunctionKeys      = True
@@ -462,6 +452,12 @@ showMainWindow          = True
 #
 quitOrcaNoConfirmation  = False
 
+# If True, always present locking key state changes; if False, never present
+# locking key state changes. If None, make the decision based on Orca's normal
+# behavior.
+#
+presentLockingKeys = None
+
 # Whether the user wants tooltips presented or not.
 #
 presentToolTips = False
